@@ -330,7 +330,7 @@ static void
 totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 {
 	GtkWidget *image;
-	gchar *id;
+	const gchar *id;
 
 	if (state == emb->state)
 		return;
@@ -348,7 +348,7 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 			(TOTEM_TIME_LABEL (emb->fs->time_label), 0, 0);
 		if (emb->href_uri != NULL && emb->hidden == FALSE) {
 			gdk_window_set_cursor
-				(GTK_WIDGET (emb->bvw)->window,
+				(gtk_widget_get_window (GTK_WIDGET (emb->bvw)),
 				 emb->cursor);
 		}
 		break;
@@ -361,7 +361,7 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 		totem_statusbar_set_text (emb->statusbar, _("Playing"));
 		if (emb->href_uri == NULL && emb->hidden == FALSE) {
 			gdk_window_set_cursor
-				(GTK_WIDGET (emb->bvw)->window,
+				(gtk_widget_get_window (GTK_WIDGET (emb->bvw)),
 				 NULL);
 		}
 		break;
@@ -402,7 +402,6 @@ totem_embedded_open_internal (TotemEmbedded *emb,
 			      gboolean start_play,
 			      GError **error)
 {
-	GError *err = NULL;
 	gboolean retval;
 	const char *uri;
 
@@ -428,37 +427,12 @@ totem_embedded_open_internal (TotemEmbedded *emb,
 
 	bacon_video_widget_set_logo_mode (emb->bvw, FALSE);
 
-	/* FIXME: remove |err| and rely on async on_error? */
-	g_message ("BEFORE _open");
-	retval = bacon_video_widget_open (emb->bvw, uri, emb->current_subtitle_uri, &err);
-	g_message ("AFTER _open (ret: %d)", retval);
-	if (retval == FALSE)
-	{
-		GError *errint;
-
-		/* FIXME we haven't even started sending yet! */
-		//g_signal_emit (emb, signals[STOP_STREAM], 0);
-
-		totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
-		totem_embedded_set_error_logo (emb, NULL);
-
-		errint = g_error_new (TOTEM_EMBEDDED_ERROR_QUARK,
-				      TOTEM_EMBEDDED_OPEN_FAILED,
-				      _("Totem could not play '%s'"),
-				     emb->current_uri);
-
-		totem_embedded_set_error (emb, err->code, err->message);;
-
-		g_propagate_error (error, err);
-
-		totem_embedded_set_pp_state (emb, FALSE);
-	} else {
-		/* FIXME we shouldn't even do that here */
-		if (start_play)
-			totem_embedded_play (emb, NULL);
-		else
-			totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), TRUE);
-	}
+	retval = bacon_video_widget_open (emb->bvw, uri, emb->current_subtitle_uri, NULL);
+	/* FIXME we shouldn't even do that here */
+	if (start_play)
+		totem_embedded_play (emb, NULL);
+	else
+		totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), TRUE);
 
 	totem_embedded_update_menu (emb);
 	if (emb->href_uri != NULL)
@@ -527,6 +501,8 @@ totem_embedded_do_command (TotemEmbedded *embedded,
 {
 	g_return_val_if_fail (command != NULL, FALSE);
 
+	g_message ("totem_embedded_do_command: %s", command);
+
 	if (strcmp (command, TOTEM_COMMAND_PLAY) == 0) {
 		return totem_embedded_play (embedded, error);
 	}
@@ -550,6 +526,9 @@ totem_embedded_set_href (TotemEmbedded *embedded,
 			 const char *target,
 			 GError *error)
 {
+	g_message ("totem_embedded_set_href %s (target: %s)",
+		   href_uri, target);
+
 	g_free (embedded->href_uri);
 	g_free (embedded->target);
 
@@ -559,7 +538,7 @@ totem_embedded_set_href (TotemEmbedded *embedded,
 		g_free (embedded->href_uri);
 		embedded->href_uri = NULL;
 		gdk_window_set_cursor
-			(GTK_WIDGET (embedded->bvw)->window, NULL);
+			(gtk_widget_get_window (GTK_WIDGET (embedded->bvw)), NULL);
 	}
 
 	if (target != NULL) {
@@ -692,6 +671,8 @@ totem_pl_item_free (gpointer data, gpointer user_data)
 static gboolean
 totem_embedded_clear_playlist (TotemEmbedded *emb, GError *error)
 {
+	g_message ("totem_embedded_clear_playlist");
+
 	g_list_foreach (emb->playlist, (GFunc) totem_pl_item_free, NULL);
 	g_list_free (emb->playlist);
 
@@ -718,7 +699,8 @@ totem_embedded_add_item (TotemEmbedded *embedded,
 {
 	TotemPlItem *item;
 
-	g_message ("totem_embedded_add_item: %s", uri);
+	g_message ("totem_embedded_add_item: %s (base: %s title: %s subtitle: %s)",
+		   uri, base_uri, title, subtitle);
 
 	item = g_slice_new0 (TotemPlItem);
 	item->uri = g_strdup (uri);
@@ -776,7 +758,7 @@ totem_embedded_set_fullscreen (TotemEmbedded *emb,
 		/* Move the fullscreen window to the screen where the
 		 * video widget currently is */
 		monitor = gdk_screen_get_monitor_at_window (gtk_widget_get_screen (GTK_WIDGET (emb->bvw)),
-							    GTK_WIDGET (emb->bvw)->window);
+							    gtk_widget_get_window (GTK_WIDGET (emb->bvw)));
 		gdk_screen_get_monitor_geometry (gtk_widget_get_screen (GTK_WIDGET (emb->bvw)),
 						 monitor, &rect);
 		gtk_window_move (GTK_WINDOW (emb->fs_window), rect.x, rect.y);
@@ -841,6 +823,8 @@ static gboolean
 totem_embedded_close_stream (TotemEmbedded *emb,
 			     GError *error)
 {
+	g_message ("totem_embedded_close_stream");
+
 	if (!emb->is_browser_stream)
 		return TRUE;
 
@@ -913,7 +897,8 @@ totem_embedded_set_local_file (TotemEmbedded *emb,
 {
 	char *file_uri;
 
-	g_message ("Setting the current path to %s", path);
+	g_message ("Setting the current path to %s (uri: %s base: %s)",
+		   path, uri, base_uri);
 
 	totem_embedded_clear_playlist (emb, NULL);
 
@@ -935,6 +920,8 @@ totem_embedded_set_local_cache (TotemEmbedded *emb,
 {
 	int fd;
 
+	g_message ("totem_embedded_set_local_cache: %s", path);
+
 	/* FIXME Should also handle playlists */
 	if (!emb->is_browser_stream)
 		return TRUE;
@@ -962,13 +949,13 @@ totem_embedded_set_playlist (TotemEmbedded *emb,
 			     GError **error)
 {
 	char *file_uri;
-	char *tmpfile;
+	char *tmp_file;
 	GError *err = NULL;
 	GFile *src, *dst;
 	int fd;
 
-	g_message ("Setting the current playlist to %s (base: %s)",
-		   path, base_uri);
+	g_message ("Setting the current playlist to %s (uri: %s base: %s)",
+		   path, uri, base_uri);
 
 	totem_embedded_clear_playlist (emb, NULL);
 
@@ -976,7 +963,7 @@ totem_embedded_set_playlist (TotemEmbedded *emb,
 	 * parse from memory or 
 	 * https://bugzilla.gnome.org/show_bug.cgi?id=598702 is fixed */
 	fd = g_file_open_tmp ("totem-browser-plugin-playlist-XXXXXX",
-			      &tmpfile,
+			      &tmp_file,
 			      &err);
 	if (fd < 0) {
 		g_warning ("Couldn't open temporary file for playlist: %s",
@@ -985,18 +972,18 @@ totem_embedded_set_playlist (TotemEmbedded *emb,
 		return TRUE;
 	}
 	src = g_file_new_for_path (path);
-	dst = g_file_new_for_path (tmpfile);
+	dst = g_file_new_for_path (tmp_file);
 	if (g_file_copy (src, dst, G_FILE_COPY_OVERWRITE | G_FILE_COPY_TARGET_DEFAULT_PERMS, NULL, NULL, NULL, &err) == FALSE) {
 		g_warning ("Failed to copy playlist '%s' to '%s': %s",
-			   path, tmpfile, err->message);
+			   path, tmp_file, err->message);
 		g_error_free (err);
 		g_object_unref (src);
 		g_object_unref (dst);
-		g_free (tmpfile);
+		g_free (tmp_file);
 		close (fd);
 		return TRUE;
 	}
-	g_free (tmpfile);
+	g_free (tmp_file);
 
 	file_uri = g_file_get_uri (dst);
 
@@ -1213,19 +1200,21 @@ popup_menu_position_func (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, Gt
 	GtkWidget *widget = GTK_WIDGET (button);
 	GtkRequisition menu_req;
 	GtkTextDirection direction;
+	GtkAllocation allocation;
 
 	gtk_widget_size_request (GTK_WIDGET (menu), &menu_req);
 
 	direction = gtk_widget_get_direction (widget);
 
-	gdk_window_get_origin (widget->window, x, y);
-	*x += widget->allocation.x;
-	*y += widget->allocation.y;
+	gdk_window_get_origin (gtk_widget_get_window (widget), x, y);
+	gtk_widget_get_allocation (widget, &allocation);
+	*x += allocation.x;
+	*y += allocation.y;
 
 	if (direction == GTK_TEXT_DIR_LTR)
-		*x += MAX (widget->allocation.width - menu_req.width, 0);
-	else if (menu_req.width > widget->allocation.width)
-		*x -= menu_req.width - widget->allocation.width;
+		*x += MAX (allocation.width - menu_req.width, 0);
+	else if (menu_req.width > allocation.width)
+		*x -= menu_req.width - allocation.width;
 
 	/* This might not work properly if the popup button is right at the
 	 * top of the screen, but really, what are the chances */
@@ -1253,7 +1242,7 @@ on_popup_button_toggled (GtkToggleButton *button, TotemEmbedded *emb)
 
 	menu = GTK_MENU (gtk_builder_get_object (emb->menuxml, "menu"));
 
-	if (gtk_toggle_button_get_active (button) && !GTK_WIDGET_VISIBLE (menu)) {
+	if (gtk_toggle_button_get_active (button) && !gtk_widget_get_visible (GTK_WIDGET (menu))) {
 		/* we get here only when the menu is activated by a key
 		 * press, so that we can select the first menu item */
 		popup_menu_over_arrow (button, menu, NULL);
@@ -1270,7 +1259,7 @@ on_popup_button_button_pressed (GtkToggleButton *button,
 		GtkMenu *menu;
 
 		menu = GTK_MENU (gtk_builder_get_object (emb->menuxml, "menu"));
-		if (!GTK_WIDGET_VISIBLE (menu)) {
+		if (!gtk_widget_get_visible (GTK_WIDGET (menu))) {
 			popup_menu_over_arrow (button, menu, event);
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 		} else {
@@ -1284,6 +1273,16 @@ on_popup_button_button_pressed (GtkToggleButton *button,
 	return FALSE;
 }
 
+static void
+on_popup_menu_unmap (GtkWidget *menu,
+		     TotemEmbedded *emb)
+{
+	GtkWidget *button;
+
+	button = GTK_WIDGET (gtk_builder_get_object (emb->xml, "popup_button"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
+}
+
 static char *
 resolve_redirect (const char *old_mrl, const char *mrl)
 {
@@ -1429,7 +1428,7 @@ on_video_button_press_event (BaconVideoWidget *bvw,
 			totem_interface_error (_("An error occurred"), emb->error->message, (GtkWindow *) (emb->window));
 			g_error_free (emb->error);
 			emb->error = NULL;
-		} else if (!GTK_WIDGET_VISIBLE (menu)) {
+		} else if (!gtk_widget_get_visible (GTK_WIDGET (menu))) {
 			g_message ("emitting signal");
 			g_signal_emit (emb, signals[BUTTON_PRESS], 0,
 				       event->time,
@@ -1752,6 +1751,7 @@ totem_embedded_construct (TotemEmbedded *emb,
 {
 	GtkWidget *child, *container, *image;
 	GtkWidget *popup_button;
+	GtkWidget *menu;
 	BvwUseType type;
 	GError *err = NULL;
 	GConfClient *gc;
@@ -1761,7 +1761,6 @@ totem_embedded_construct (TotemEmbedded *emb,
 					 GTK_WINDOW (emb->window), emb);
 	g_assert (emb->xml);
 
-
 	if (xid != 0) {
 		g_assert (!emb->hidden);
 
@@ -1969,6 +1968,10 @@ totem_embedded_construct (TotemEmbedded *emb,
 					     GTK_WINDOW (emb->window), emb);
 	g_assert (emb->menuxml);
 
+	menu = GTK_WIDGET (gtk_builder_get_object (emb->menuxml, "menu"));
+	g_signal_connect (G_OBJECT (menu), "unmap",
+			  G_CALLBACK (on_popup_menu_unmap), emb);
+
 	/* Set the logo and the button glow */
 	if (!emb->hidden && emb->autostart == FALSE) {
 		totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), TRUE);
@@ -2246,18 +2249,16 @@ int main (int argc, char **argv)
 
 		env = g_getenv ("TOTEM_EMBEDDED_GDB");
 		if (env && g_ascii_strtoull (env, NULL, 10) == 1) {
-			char *gdbargv[6];
-			char pid[32];
+			char *gdbargv[4];
+			char *gdb;
 			GError *gdberr = NULL;
 			int gdbargc = 0;
 
-			g_snprintf (pid, sizeof (pid), "%d", getpid ());
+			gdb = g_strdup_printf ("gdb %s %d", argv[0], getpid ());
 
-			gdbargv[gdbargc++] = "/usr/bin/xterm";
+			gdbargv[gdbargc++] = "/usr/bin/gnome-terminal";
 			gdbargv[gdbargc++] = "-e";
-			gdbargv[gdbargc++] = "gdb";
-			gdbargv[gdbargc++] = argv[0];
-			gdbargv[gdbargc++] = pid;
+			gdbargv[gdbargc++] = gdb;
 			gdbargv[gdbargc++] = NULL;
 
 			if (!g_spawn_async (NULL,
