@@ -855,6 +855,7 @@ totemPlugin::ViewerReady ()
 #ifdef TOTEM_NARROWSPACE_PLUGIN
 	/* Tell the viewer it has an href */
 	if (mHref) {
+		Dm("SetHref in ViewerReady");
 		dbus_g_proxy_call_no_reply (mViewerProxy,
 					    "SetHref",
 					    G_TYPE_STRING, mHref,
@@ -1243,6 +1244,27 @@ totemPlugin::TickCallback (DBusGProxy *proxy,
 		}
 	}
 #endif /* TOTEM_GMP_PLUGIN */
+
+#ifdef TOTEM_NARROWSPACE_PLUGIN
+        if (!plugin->mNPObjects[ePluginScriptable].IsNull ()) {
+                NPObject *object = plugin->mNPObjects[ePluginScriptable];
+                totemNarrowSpacePlayer *scriptable = static_cast<totemNarrowSpacePlayer*>(object);
+                switch (plugin->mState) {
+		case TOTEM_STATE_PLAYING:
+		case TOTEM_STATE_PAUSED:
+			scriptable->mPluginState = totemNarrowSpacePlayer::eState_Playable;
+			break;
+		case TOTEM_STATE_STOPPED:
+			if (scriptable->mPluginState == totemNarrowSpacePlayer::eState_Playable)
+				scriptable->mPluginState = totemNarrowSpacePlayer::eState_Complete;
+			else
+				scriptable->mPluginState = totemNarrowSpacePlayer::eState_Waiting;
+			break;
+		default:
+			scriptable->mPluginState = totemNarrowSpacePlayer::eState_Waiting;
+		}
+	}
+#endif /* TOTEM_NARROWSPACE_PLUGIN */
 }
 
 /* static */ void
@@ -1597,6 +1619,30 @@ totemPlugin::SetURL (const char* aURL)
 
 #ifdef TOTEM_NARROWSPACE_PLUGIN
 
+void
+totemPlugin::SetURL (const NPString& aURL)
+{
+        g_free (mSrcURI);
+
+	/* If |src| is empty, don't resolve the URI! Otherwise we may
+	 * try to load an (probably iframe) html document as our video stream.
+	 */
+	if (!aURL.UTF8Characters || !aURL.UTF8Length) {
+              mSrcURI = NULL;
+              return;
+        }
+
+        mSrcURI = g_strndup (aURL.UTF8Characters, aURL.UTF8Length);
+
+        UnsetStream ();
+
+        if (mAutoPlay) {
+                RequestStream (true); //FIXME
+        } else {
+                mWaitingForButtonPress = true;
+        }
+}
+
 bool
 totemPlugin::SetQtsrc (const char* aURL)
 {
@@ -2347,6 +2393,7 @@ totemPlugin::StreamAsFile (NPStream *stream,
 	gboolean retval = TRUE;
 	GError *error = NULL;
 	if (mIsPlaylist) {
+		Dm("Calling SetPlaylist in StreamAsFile");
 		retval = dbus_g_proxy_call (mViewerProxy,
 					    "SetPlaylist",
 					    &error,
@@ -2361,6 +2408,7 @@ totemPlugin::StreamAsFile (NPStream *stream,
 	 * completely in the cache.)
 	 */
 	else if (mBytesStreamed == 0) {
+		Dm("Calling SetLocalFile from ViewerReady");
 		retval = dbus_g_proxy_call (mViewerProxy,
 					    "SetLocalFile",
 					    &error,
@@ -2409,7 +2457,13 @@ totemPlugin::URLNotify (const char *url,
 		        NPReason reason,
 		        void *notifyData)
 {
-	D ("URLNotify URL '%s' reason %d", url ? url : "", reason);
+	const char *reasons[] = {
+		"Base (undefined)",
+		"Done",
+		"Network error",
+		"User break"
+	};
+	D ("URLNotify URL '%s' reason %d (%s)", url ? url : "", reason, reasons[reason]);
 
 	/* If we get called when we expect a stream,
 	 * it means that the stream failed.
